local	MainLayer = {}

local	Cover = import(".Cover.Cover")

local	PlayerMaster = import(".Components.PlayerMaster")
local	PleditMaster = import(".Components.PleditMaster")

local	TaskBar = import(".TaskBar.TaskBar")
local	MiniBrowser = import(".MiniBrowser.MiniBrowser")


MainLayer = class("MainLayer", cc.Layer)

function MainLayer:ctor(...)
	cc.Layer.ctor(self, ...)
	
	
	do
		local	cover = Cover:create()
		cover:setPosition(0, 24)
		cover:setName("cover")
		cover:setVisible(false)
		self:addChild(cover)
	end
	
	do
		local	taskbar = TaskBar:create()
		taskbar:setContentSize(cc.size(self:getContentSize().width, 24))
		taskbar:setName("taskbar")
		self:addChild(taskbar)
	end
	
	do
		local	playerMaster = PlayerMaster:create()
		
		playerMaster:setPosition(
			love.math.random(0, self:getContentSize().width * 0.25),
			love.math.random(0, self:getContentSize().height * 0.25)
		)
		
		playerMaster:setName("player")
		playerMaster:setVisible(false)
		playerMaster.m_enableCtrlAlt4 = true
		self:addChild(playerMaster)
	end
	
	do
		local	pleditMaster = PleditMaster:create()
	
		pleditMaster:setPosition(
			love.math.random(0, self:getContentSize().width * 0.25),
			love.math.random(0, self:getContentSize().height * 0.25)
		)
		pleditMaster:setName("pledit")
		pleditMaster:setVisible(false)
		pleditMaster.m_enableCtrlAlt4 = true
		self:addChild(pleditMaster)
	end
	
	local	miniBrowsers = {
			{
				"mbPistonsource",
				"http://pistonsource.agargara.com/",
				"res/ps040mp3/[Azargabra] - (PS040) Zebrafish [2014] (mp3)/00 [Azargabra] - Zebrafish.nfo",
			},
			{
				"mbAgargara",
				"http://twitter.com/agargara",
				"res/AudioTest/Agargara.txt"
			},
			{
				"mbZebra",
				"http://twitter.com/pipluplupsalad",
				"res/AudioTest/Zebra.txt"
			},
			{
				"mbYamahara",
				"http://twitter.com/yamahara",
				"res/AudioTest/Yamahara.txt"
			},
			{
				"mbPixel",
				"http://twitter.com/oxizn",
				"res/AudioTest/Pixel.txt"
			},
			{
				"mbPxtone",
				"http://twitter.com/oxizm",
				"res/AudioTest/Pxtone.txt"
			},
			{
				"mbLibs",
				"http://love2d.org",
				"res/AudioTest/Libs.txt"
			},
	}
	
	for index, mb in ipairs(miniBrowsers) do
		local	miniBrowser = MiniBrowser:create()
		miniBrowser:setPosition(
			love.math.random(0, self:getContentSize().width * 0.25),
			love.math.random(0, self:getContentSize().height * 0.25)
		)
		if mb[1] then miniBrowser:setName(mb[1]) end
		if mb[2] then miniBrowser:setUrl(mb[2]) end
		if mb[3] then miniBrowser:setFilename(mb[3]) end
		miniBrowser:setVisible(false)
		miniBrowser.m_enableCtrlAlt4 = true
		self:addChild(miniBrowser)
	end
	
	do
		local	eventKeyboard = cc.EventKeyboard:create()
		
		eventKeyboard.keyPressed = self.keyPressed
		eventKeyboard.keyReleased = self.keyReleased
		
		self:addEventListener(
			eventKeyboard
		)
	end
	
	self:scheduleUpdate()
end


function MainLayer:update(dt)
	
	
	local	audioManager = self:getScene():getAudioManager()
	local	source = nil
	local	cover = self:getChildByName("cover")
	if audioManager then source = audioManager:getSource() end
	
	if cover then
		if source == nil then
			cover:setVisible(false)
		else
			cover:setVisible(true)
		end
	end
end


function MainLayer:keyPressed(
	event
	)
	local	keys = event:getKeys()
	local	key, isrepeat = event:getKey()
	
	if keys["rctrl"] or keys["lctrl"] then
		if key == "f4" and isrepeat == false then
			local	children = self:getChildren()
			for i = #children, 1, -1 do
				if children[i]:isVisible() then
					if children[i].m_enableCtrlAlt4 == true then
						children[i]:setVisible(false)
					end
					break
				end
			end
			return true
		end
	end
	
end


function MainLayer:keyReleased(
	event
	)
	
end

function MainLayer:onReload()
	
	local	pledit = self:getChildByName("pledit")
	if pledit then pledit = pledit:getChildByName("pledit") end
	if pledit then pledit:onReload() end
end


return MainLayer
