
local	kconv = import("...Libs.kconv")
--local	utf8 = import("...Libs.utf8")

local	Slave = import("..Components.Slave")
local	MiniBrowser_MainTop = import(".MiniBrowser_MainTop")
local	MiniBrowser_MainMiddle = import(".MiniBrowser_MainMiddle")
local	MiniBrowser_MainBottom = import(".MiniBrowser_MainBottom")



local	MiniBrowser = {}
MiniBrowser = class("MiniBrowser", Slave)

function MiniBrowser:ctor(...)
	Slave.ctor(self, ...)
	
	self.m_filename = nil
	self.m_fontSize = 14
	self.m_url = ""
	
	self.m_cellMin = cc.p(1, 2)
	self.m_cell = cc.p(self.m_cellMin.x, self.m_cellMin.y)
	self.m_cellSize = cc.size(25, 29)
	
	self:setCell()
	
--	self:setContentSize(cc.size(275, 116))
	
--	self:setBackGroundColor(cc.c3b(0xff, 0xff, 0xff))
--	self:setBackGroundColorType(1)
	
	do
		local	mainTop = MiniBrowser_MainTop:create()
		mainTop:setName("mainTop")
		self:addChild(mainTop)
	end
	
	do
		local	mainMiddle = MiniBrowser_MainMiddle:create()
		mainMiddle:setName("mainMiddle")
		self:addChild(mainMiddle)
	end
	
	do
		local	mainBottom = MiniBrowser_MainBottom:create()
		mainBottom:setName("mainBottom")
		mainBottom:setPosition(0, self:getContentSize().height - mainBottom:getContentSize().height)
		self:addChild(mainBottom)
	end
	
	do
		local	eventMouse = cc.EventMouse:create()
		eventMouse.onMouseMove = self.onMouseMove
		eventMouse.onMouseDown = self.onMouseDown
		eventMouse.onMouseUp = self.onMouseUp
--		eventMouse.onScroll = self.onScroll
		
		self:addEventListener(
			eventMouse
		) 
	end
	
	self:cleanup()
	
	if self:getScrollView() then
		self:getScrollView():setBackGroundColorType(2)
		self:getScrollView():setBackGroundColor(cc.c3b(0, 0, 62))
	end
	
--	self:scheduleUpdate()
end


function MiniBrowser:update(dt)
	
--	if self:getScrollView() then
--		if self:isForeground() then
--			self:getScrollView():setBackGroundColor(cc.c3b(0, 0, 62))
--		else
--			self:getScrollView():setBackGroundColor(cc.c3b(0, 0, 62))
--		end
--	end
end

function MiniBrowser:onMouseMove(
	event
	)
	if event:isCapture(self) then
		local	object = self
		if object == nil then object = self end
		local	position = cc.pAdd(object:getPosition(), event:getDelta())
		object:setPosition(position)
	end
end


function MiniBrowser:onMouseDown(
	event
	)
	if self:isHover(event:getLocation()) then
		self:setForeground()
		event:setCapture(self)
		return true
	end
end


function MiniBrowser:onMouseUp(
	event
	)
	if event:isCapture(self) then
		event:removeCapture()
		return true
	end
end



function MiniBrowser:setCell(cell)
	
	-- Zw
	self.m_cell = cell or self.m_cell
	
	self:setContentSize(
		cc.size(
			250 + self.m_cell.x * self.m_cellSize.width,
			 58 + self.m_cell.y * self.m_cellSize.height
		)
	)
	
	local	children = self:getChildren()
	for key, child in ipairs(children) do
		if type(child.onResize) == "function" then
			child:onResize()
		end
	end
end


function MiniBrowser:getScrollView()
	local	mainMiddle = self:getChildByName("mainMiddle")
	if mainMiddle then
		return mainMiddle:getChildByName("scrollView")
	end
end


function MiniBrowser:setFilename(filename, fontSize)
	fontSize = fontSize or self.m_fontSize
	if self.m_filename ~= filename or self.m_fontSize ~= fontSize then
		self.m_filename = filename
		self.m_fontSize = fontSize
		self:reload(self.m_filename, self.m_fontSize)
	end
end


function MiniBrowser:getFontSize()
	return self.m_fontSize
end


function MiniBrowser:setFontSize(fontSize)
	if self.m_fontSize ~= fontSize then
		self.m_fontSize = fontSize
		self:reload(self.m_filename, self.m_fontSize)
	end
end


function MiniBrowser:setUrl(url)
	self.m_url = url
end

function MiniBrowser:getUrl()
	return self.m_url
end


function MiniBrowser:reload(filename, fontSize)
	local	scrollView = self:getScrollView()
	if scrollView then
		local	inner = scrollView:getInnerContainer()
		local	x, y = 0, 0
		
		inner:removeAllChildren()
		if filename then
			for line in love.filesystem.lines(filename) do
				local	line_utf8 = kconv.kconvert(line, "sjis", "utf-8")
				do
					local	text = cc.Label:create()
					local	url = string.match(line_utf8, '(%w+://[^%s]+)')
					
					text:createWithTTF(line_utf8, "res/IPAexfont00301/ipaexg.ttf", fontSize)
					text:setAnchorPoint(cc.p(0, 0))
					text:setPosition(cc.p(0, y))
					
					if url then
						text:setTextColor(cc.c3b(0xf0, 0x80, 0x00))
						text.m_url = url
						
						do
							local	button = ccui.Button:create()
							button:setContentSize(text:getContentSize())
							button:setPosition(text:getContentSize().width * 0.5, text:getContentSize().height * 0.5)
							
							button:setMouseEnabled(
								true,
								{
									onEnter = function(
										button,
										event
										)
										
										if love.mouse.hasCursor() then
											love.mouse.setCursor(love.mouse.getSystemCursor("hand"))
										end
										button:getParent():setTextColor(cc.c3b(0xff, 0x00, 0xff))
									end,
									onLeave = function(
										button,
										event
										)
										if love.mouse.hasCursor() then
											love.mouse.setCursor()
										end
										button:getParent():setTextColor(cc.c3b(0xf0, 0x80, 0x00))
									end,
									onClick = function(
										button,
										event
										)
										love.system.openURL(button:getParent().m_url)
									end
								}
							)
							text:addChild(button)
						end
						
					else
						text:setTextColor(cc.c3b(0x00, 0xf0, 0x80))
					end
					
					y = y + text:getContentSize().height
					if text:getContentSize().width > x then x = text:getContentSize().width end
					inner:addChild(text)
				end
			end
		end
		scrollView:setInnerContainerSize(cc.size(x, y))
	end
end


--------------------------------
--  file:lines() bug?
--------------------------------
--function MiniBrowser:reload(filename, fontSize)
--	local	scrollView = self:getScrollView()
--	if scrollView then
--		local	inner = scrollView:getInnerContainer()
--		local	x, y = 0, 0
--		
--		inner:removeAllChildren()
--		if filename then
--			local	file = love.filesystem.newFile(filename)
--			if file then
--				if file:open("r") then
--					for line in  file:lines() do
--						local	line_utf8 = kconv.kconvert(line, "sjis", "utf-8")
--						do
--							local	text = cc.Label:create()
--							
--							local	url = string.match(line_utf8, '(%w+://[^%s]+)')
--							
--							text:createWithTTF(line_utf8, "res/IPAexfont00301/ipaexg.ttf", fontSize)
--							text:setAnchorPoint(cc.p(0, 0))
--							text:setPosition(cc.p(0, y))
--							
--							if url then
--								text:setTextColor(cc.c3b(0xf0, 0x80, 0x00))
--								text.m_url = url
--								
--								do
--									local	button = ccui.Button:create()
--									button:setContentSize(text:getContentSize())
--									button:setPosition(text:getContentSize().width * 0.5, text:getContentSize().height * 0.5)
--									
--									button:setMouseEnabled(
--										true,
--										{
--											onEnter = function(
--												button,
--												event
--												)
--												
--												if love.mouse.hasCursor() then
--													love.mouse.setCursor(love.mouse.getSystemCursor("hand"))
--												end
--												button:getParent():setTextColor(cc.c3b(0xff, 0x00, 0xff))
--											end,
--											onLeave = function(
--												button,
--												event
--												)
--												if love.mouse.hasCursor() then
--													love.mouse.setCursor()
--												end
--												button:getParent():setTextColor(cc.c3b(0xf0, 0x80, 0x00))
--											end,
--											onClick = function(
--												button,
--												event
--												)
--												love.system.openURL(button:getParent().m_url)
--											end
--										}
--									)
--									text:addChild(button)
--								end
--								
--							else
--								text:setTextColor(cc.c3b(0x00, 0xf0, 0x80))
--							end
--							
--							y = y + text:getContentSize().height
--							if text:getContentSize().width > x then x = text:getContentSize().width end
--							
--							inner:addChild(text)
--						end
--					end
--				end
--				file:close()
--			end
--		end
--		scrollView:setInnerContainerSize(cc.size(x, y))
--	end
--end
--

return MiniBrowser
