
local	complex = import("..Libs.complex")
local	luafft = import("..Libs.luafft")


-- なんちゃてオーディオマネージャー
local	Path = import("..Utils.Path")

local	AudioManager = {}
AudioManager = class("AudioManager")


function AudioManager:ctor()
	self.m_source = nil
	self.m_soundData = nil
	self.m_soundStatus = {}
	self.m_soundStatus.bitDepth = 16
	self.m_soundStatus.sampleRate = 44100
	
	self.m_sample = 0
	self.m_second = 0
	self.m_file = nil
	self.m_filename = ""
	self.m_name = ""
	
	self.m_spectrum = {}
end


function AudioManager:setFile(
	filename,
	file
	)
	self.m_filename = filename
	self.m_file = file
	if type(self.m_filename) == "string" then
		self.m_name = Path:removeExtension(Path:stripPath(self.m_filename))
	else
		self.m_name = ""
	end
end


--function AudioManager:getFile()
--	return self.m_file
--end


function AudioManager:getName()
	return self.m_name
end


function AudioManager:readFile(
	file
	)
	self.m_soundData = love.sound.newSoundData(file)
	if self.m_soundData then
		self.m_soundStatus.bitDepth = self.m_soundData:getBitDepth()
		self.m_soundStatus.channels = self.m_soundData:getChannels()
		self.m_soundStatus.duration = self.m_soundData:getDuration()
		self.m_soundStatus.sampleCount = self.m_soundData:getSampleCount()
		self.m_soundStatus.sampleRate = self.m_soundData:getSampleRate()
		
		self.m_source = love.audio.newSource(self.m_soundData)
	end
end


function AudioManager:getSource()
	return self.m_source
end


function AudioManager:getSoundData()
	return self.m_soundData
end


function AudioManager:getSoundStatus()
	return self.m_soundStatus
end


function AudioManager:getSecond()
	return self.m_second
end


function AudioManager:getSample()
	return self.m_sample
end


function AudioManager:set()
--	if type(self.m_file) ~= nil and self.m_soundData == nil then
--		self:readFile(self.m_file)
--	end
	if self.m_soundData == nil then
		local	file = self.m_file
		if file == nil and self.m_filename ~= "" then file = self.m_filename end
		if file then self:readFile(file) end
	end
end

function AudioManager:play()
	self:set()
	if self.m_source then
		self.m_source:play()
	end
end


function AudioManager:stop()
--	self:set()
	if self.m_source then
		self.m_source:stop()
--		self.m_second = 0
--		self.m_sample = 0
	end
end


function AudioManager:pause()
--	self:set()
	if self.m_source then
		if self.m_source:isPaused() then
			self.m_source:resume()
		else
			self.m_source:pause()
		end
	end
end

function AudioManager:getSpectrum()
	return self.m_spectrum
end

function AudioManager:update(dt)
	local	devide = function(list, factor)
		for i,v in ipairs(list) do list[i] = list[i] * factor end
	end
	self.m_spectrum = {}
	if self.m_source then
		self.m_second = self.m_source:tell()
		self.m_sample = self.m_source:tell("samples")
		
		local	size = 1024
		local	list = {}
		for pos = self.m_sample, self.m_sample + (size - 1) do
			if pos >= self.m_soundData:getSampleCount() then pos = self.m_soundData:getSampleCount() - 1 end
			local	samplepos = pos * self.m_soundStatus.channels
			list[#list + 1] = complex.new(self.m_soundData:getSample(samplepos), 0)
		end
		self.m_spectrum = luafft.fft(list, false)
		devide(self.m_spectrum, 10)
	end
end


return AudioManager
