
-- import
local	Player = import(".Entities.Player")
local	Block = import(".Entities.Block")

local	bump = import(".lib.bump")

local	Stage = {}

Stage = class("Stage", ccui.ScrollView)

local	width = 4000
local	height = 2000

function bump.collision(obj1, obj2, dx, dy)
	
	obj1:bumpCollision(obj2, dx, dy)
	obj2:bumpCollision(obj1, -dx, -dy)
end


function bump.shouldCollide(obj1, obj2)
	
	return obj1:bumpShouldCollide(obj2) or obj2:bumpShouldCollide(obj1)
end


function bump.getBBox(obj)
	return obj:bumpBoundingBox()
end


function Stage:ctor(...)
	ccui.ScrollView.ctor(self, ...)
	
	self.m_bump = bump
	
	self:setInnerContainerSize(cc.size(width, height))
	
	-- walls & ceiling
	do
		local	rects = {
			cc.rect         (0,  0, width,          32),
			cc.rect(         0, 32,    32, height - 64),
			cc.rect(width - 32, 32,    32, height - 64)
		}
		for i = 1, #rects do
			local	block = Block:create()
			block:setPosition(cc.p(cc.rectGetMinX(rects[i]), cc.rectGetMinY(rects[i])))
			block:setContentSize(cc.size(rects[i].width, rects[i].height))
			self:addChild(block)
		end
	end
	
	-- tiled floor
	local	tilesOnFloor = 40
	for i = 0, tilesOnFloor - 1 do
		local	block = Block:create()
		block:setPosition(cc.p(i * width / tilesOnFloor, height - 32))
		block:setContentSize(cc.size(width / tilesOnFloor, 32))
		self:addChild(block)
	end
	
	-- random blocks
	for i = 1, 100 do
		local	block = Block:create()
		block:setPosition(cc.p(math.random(100, width - 200), math.random(100, height - 150)))
		block:setContentSize(cc.size(math.random(32, 100), math.random(32, 100)))
		self:addChild(block)
	end
	
	do
		local	player = Player:create()
		player:setPosition(cc.p(60, 60))
		self:addChild(player)
	end
	
	self:scheduleUpdate()
end

function Stage:getBump()
	return self.m_bump
end


function Stage:update(dt)
	local	viewport = self:getViewport()
	local	maxdt = 0.1
	
	dt = math.min(dt, maxdt)
	
	local updateEntity = function(entity) entity:bumpUpdate(dt, maxdt) end
	self.m_bump.each(updateEntity, viewport.x, viewport.y, viewport.width, viewport.height)
	self.m_bump.collide(viewport.x, viewport.y, viewport.width, viewport.height)
end


return Stage

