local	EventMouseManager = {}

EventMouseManager = class("cc.EventMouseManager")


function EventMouseManager:ctor()
	local	x, y = love.mouse.getX(), love.mouse.getY()
	
	x = x or 0
	y = y or 0
	
	self.m_location = cc.p(x, y)
	self.m_previousLocation = self.m_location
	self.m_startLocation = nil
	self.m_buttons = {}
	self.m_button = nil
	
	self.m_capture = nil
	self.m_list = {}
	self.m_delta = 0
end


function EventMouseManager:addNode(
	node,
	event
	)
	
	event.m_manager = self
	self.m_list[node] = event
end


function EventMouseManager:removeNode(
	node
	)
	self.m_list[node] = nil
end


function EventMouseManager:getLocation()
	return self.m_location
end


function EventMouseManager:getPreviousLocation()
	return self.m_previousLocation
end


function EventMouseManager:getStartLocation()
	return self.m_startLocation
end


function EventMouseManager:getScroll()
	return self.m_scroll
end

function EventMouseManager:getButton()
	return self.m_button
end


function EventMouseManager:getButtons()
	return self.m_buttons
end


function EventMouseManager:getDelta()
	return self.m_delta
end


function EventMouseManager:getCapture(
	-- 
	)
	return self.m_capture
end


function EventMouseManager:setCapture(
	-- 
	capture
	)
	self.m_capture = capture
end


function EventMouseManager:removeCapture(
	-- 
	)
	self.m_capture = nil
end


function EventMouseManager:callEvent(
	callname
	)
	local	capture = self.m_capture
	
	if capture then
		local	event = self.m_list[capture]
		if type(event[callname]) == "function" then
			event[callname](capture, event)
		end
	else
		local	tbls = {}
		table.walk(
			self.m_list,
			function(event, node)
				table.insert(tbls, { node = node, event = event })
			end
		)
		table.sort(
			tbls,
			function(a, b)
				return a.node:getDrawOrder() > b.node:getDrawOrder()
			end
		)
		for index, tbl in ipairs(tbls) do
			if type(tbl.event[callname]) == "function" and tbl.node:isVisible() then
--			if type(tbl.event[callname]) == "function" and tbl.node.m_visible then
				local	bl = tbl.event[callname](tbl.node, tbl.event)
				if bl then break end
			end
		end
--		for node, event in pairs(self.m_list) do
--			if type(event[callname]) == "function" then
--				event[callname](node, event)
--			end
--		end
	end
end


function EventMouseManager:mousemoved(x, y, dx, dy)
	self.m_scroll = cc.p(0, 0)
	self.m_delta = cc.p(dx, dy)
	self.m_previousLocation = self.m_location
	self.m_location = cc.p(x, y)
	self.m_button = nil
	
	self:callEvent("onMouseMove")
end


function EventMouseManager:mousepressed(x, y, button)
	self.m_scroll = cc.p(0, 0)
	self.m_delta = cc.p(0, 0)
	self.m_previousLocation = self.m_location
	self.m_location = cc.p(x, y)
	self.m_buttons[button] = true
	self.m_button = button
	
	-- ̃Cxg
	self:callEvent("mousePressed")
	
	if button == 1 then
--	if button == "l" then
		self.m_startLocation = cc.p(x, y)
		love.mouse.setGrabbed(true)
		self:callEvent("onMouseDown")
	end
end


function EventMouseManager:mousereleased(x, y, button)
	self.m_scroll = cc.p(0, 0)
	self.m_delta = cc.p(0, 0)
	self.m_previousLocation = self.m_location
	self.m_location = cc.p(x, y)
	self.m_buttons[button] = false
	self.m_button = button
	
	-- ̃Cxg
	self:callEvent("mouseReleased")
	
	if button == 1 then
--	if button == "l" then
		self:callEvent("onMouseUp")
		self.m_startLocation = nil
		love.mouse.setGrabbed(false)
	end
end


function EventMouseManager:wheelmoved(x, y)
	self.m_scroll = cc.p(x, y)
	self:callEvent("onScroll")
end

return EventMouseManager
