-- なんちゃってラベル
local	Node = import(".Node")

local	Label = {}
Label = class("cc.Label", Node)


function Label:ctor(...)
	Node.ctor(self, ...)
	
	self:setAnchorPoint(cc.p(0.5, 0.5))
	
	self.m_string = ""
	self.m_font = nil
	self.m_fontSize = 12
	self.m_textColor = cc.c4b(255, 255, 255, 255)
	self.m_rotation = 0
end


function Label:createWithTTF(
	string,
	filePath,
	fontSize
	)
	self.m_font = self:getDirector():loadSource("font", filePath, fontSize)
--	self.m_font:setFilter("nearest", "nearest")
	self:setString(string)
end


function Label:getTextColor()
	return self.m_textColor
end


function Label:setTextColor(
	textColor
	)
	self.m_textColor = textColor
end


function Label:setRotation(
	-- 角度をセット（angleで指定してね）
	rotation
	)
	self.m_rotation = rotation
end


function Label:getRotation()
	return self.m_rotation
end



function Label:getString()
	return self.m_string
end


function Label:setString(
	string
	)
	
	self.m_string = string
	
	local	font = nil
	if self.m_font then
		font = self.m_font
	else
		font = love.graphics.getFont()
	end
	self:setContentSize(cc.size(font:getWidth(self.m_string), font:getHeight()))
end


function Label:draw()
	local	drawSpace = self:getDrawSpace()
	local	x, y = math.floor(drawSpace.x), math.floor(drawSpace.y)
	local	w, h = self:getContentSize().width, self:getContentSize().height
	
	local	prevFont = love.graphics.getFont()
	local	prevColor = { love.graphics.getColor() }
	
	local	ap = self:getAnchorPoint()
	local	apip = self:getAnchorPointInPoints()
	local	rotate = math.angle2radian(self:getRotation())
	local	left = x + apip.x - w * ap.x
	local	top  = y + apip.y - h * ap.y
	
	love.graphics.push()
		-- 移動して
		love.graphics.translate( math.floor(left + w * ap.x),  math.floor(top + h * ap.y))
		-- 回転して
		love.graphics.rotate(rotate)
		-- 移動もとを戻す
		love.graphics.translate(-math.floor(left + w * ap.x), -math.floor(top + h * ap.y))
		
		
		if self.m_font then love.graphics.setFont(self.m_font) end
		love.graphics.setColor(self.m_textColor.r, self.m_textColor.g, self.m_textColor.b, self.m_textColor.a)
		
		love.graphics.print(self.m_string, drawSpace.x, drawSpace.y)
		
		-- 元に戻す
		love.graphics.origin()
	love.graphics.pop()
	
	love.graphics.setFont(prevFont)
	love.graphics.setColor(unpack(prevColor))
	
	Node.draw(self)
end

return Label
