-- ^m[h

local	NodeManager = import(".NodeManager")

local	Node = {}
Node = class("cc.Node")


function Node:ctor()
	-- {\z
	self.m_parent = nil
	self.m_director = nil
	self.m_scene = nil
	self.m_visible = true
	self.m_tag = nil
	self.m_name = nil
	self.m_x = 0
	self.m_y = 0
	self.m_anchorPoint = cc.p(0, 0)
	self.m_contentSize = cc.size(0, 0)
	self.m_nodeManager = NodeManager:create(self)
	self.m_eventMouseListener = nil
	self.m_localZOrder = 0
	self.m_rowid = 0
	
	self.m_drawOrder = 0
	self.m_drawSpace = cc.p(0, 0)
	
	-- o^
	self.m_entries = {}
end

function Node:getNodeManager()
	return self.m_nodeManager
end


function Node:isVisible()
	local	visible = self.m_visible
	local	node = self.m_parent
	while visible == true and node do
		visible = node.m_visible
		node = node.m_parent
	end
	return visible
end

--function Node:isVisible()
--	return self.m_visible
--end


function Node:setVisible(
	visible
	)
	self.m_visible = visible
end


function Node:getLocalZOrder()
	return self.m_localZOrder
end


function Node:setLocalZOrder(
	localZOrder
	)
	self.m_localZOrder = localZOrder
end


function Node:setName(
	name
	)
	self.m_name = name
end


function Node:getName()
	return self.m_name
end


function Node:setTag(
	tag
	)
	self.m_tag = tag
end


function Node:getTag()
	return self.m_tag
end


function Node:getParent()
	return self.m_parent
end


function Node:getContentSize()
	return self.m_contentSize
end


function Node:setContentSize(
	contentSize
	)
	self.m_contentSize = contentSize
end


function Node:setPosition(
	x,
	y
	)
	self.m_x = x
	self.m_y = y
end


function Node:getPosition()
	return cc.p(self.m_x, self.m_y)
end


function Node:setAnchorPoint(
	anchorPoint
	)
	self.m_anchorPoint = anchorPoint
end


function Node:getAnchorPoint()
	return self.m_anchorPoint
end


function Node:cleanup()
	self:getNodeManager():cleanup()
end


function Node:getAnchorPointInPoints()
	-- AJ[|Cg̉Zlpixelɕϊ
	local	contentSize = self:getContentSize()
	local	anchorPoint = self:getAnchorPoint()
	return cc.p(contentSize.width * anchorPoint.x, contentSize.height * anchorPoint.y)
end


function Node:convertToWorldSpace(
	-- [hWɕϊiWj
	point
	)
	point = point or cc.p(0, 0)
	
	local	node = self
	while node do
		local	lt = cc.pSub(node:getPosition(), node:getAnchorPointInPoints())
		point = cc.p(point.x + lt.x, point.y + lt.y)
		node = node:getParent()
	end
	return point
end


function Node:convertToWorldSpaceAR(
	-- [hWɕϊiAJ[|Cgɏ]j
	point
	)
	point = point or cc.p(0, 0)
	local	apip = self:getAnchorPointInPoints()
	return self:convertToWorldSpace(cc.p(apip.x + point.x, apip.y + point.y))
end


function Node:convertToNodeSpace(
	-- [JWɕϊij
	point
	)
	point = point or cc.p(0, 0)
	
	local	node = self
	while node do
		local	lt = cc.pSub(node:getPosition(), node:getAnchorPointInPoints())
		point = cc.p(point.x - lt.x, point.y - lt.y)
		node = node:getParent()
	end
	return point
end


function Node:convertToNodeSpaceAR(
	-- [JWɕϊiAJ[|Cgɏ]j
	point
	)
	point = point or cc.p(0, 0)
	local	apip = self:getAnchorPointInPoints()
	return self:convertToNodeSpace(cc.p(-apip.x + point.x, -apip.y + point.y))
end


function Node:getBoundingBox()
	local	lt = self:convertToWorldSpace(cc.p(0, 0))
	local	contentSize = self:getContentSize()
	-- dȂ悤-1Ă݂cc
	return cc.rect(lt.x, lt.y, contentSize.width - 1, contentSize.height - 1)
end


function Node:getDrawSpace(p)
	-- `ʒu擾idrawƂŎgpj
	p = p or cc.p(0, 0)
	return cc.pAdd(self.m_drawSpace, p)
--	return self:getNodeManager():getDrawSpace()
end


function Node:getDrawArea()
	-- `GA擾idrawƂŎgpj
	local	space = self:getDrawSpace()
	return cc.rect(space.x, space.y, self:getContentSize().width, self:getContentSize().height)
end


function Node:getStencilDrawArea()
	-- XeV`GA擾inodeManagerƂŎgpj
	local	stencilDrawArea = self:getDrawArea()
	local	node = self:getParent()
	local	director = node:getDirector()
	while node do
		if node == director then break end
		local	drawArea = node:getDrawArea()
		stencilDrawArea = cc.rectIntersection(stencilDrawArea, drawArea)
		if stencilDrawArea.width < 0 or stencilDrawArea.height < 0 then break end
		node = node:getParent()
	end
	return stencilDrawArea
end

function Node:getDrawOrder()
	-- `揇擾i}EX̑O֌WƂŎgpj
	return self.m_drawOrder
--	return self:getNodeManager():getDrawOrder()
end


function Node:getDirector()
	-- Ȃ΂Ⴄ
	if self.m_director == nil then
		local	node = self.m_parent
		while node do
			self.m_director = node.m_director
			if self.m_director then break end
			node = node.m_parent
		end
	end
	return self.m_director or cc.Director:getInstance()
end


function Node:getScene()
	-- Ȃ΂Ⴄ
	if self.m_scene == nil then
		local	node = self.m_parent
		while node do
			self.m_scene = node.m_scene
			if self.m_scene then break end
			node = node.m_parent
		end
	end
	
--	if self.m_scene == nil and self:getDirector() then
--		self.m_scene = self:getDirector():getRunScene()
--	end
	
	return self.m_scene
end


function Node:onEnter() end
function Node:onExit() end
function Node:update() end


function Node:addChild(
	child
	)
	child.m_parent = self
	self:getNodeManager():addChild(child)
	return child
end


function Node:addTo(
	parent
	)
	parent:addChild(self)
end


function Node:getChildByTag(
	-- ^OtQbg
	tag
	)
	return self:getNodeManager():getChildByTag(tag)
end


function Node:getChildByName(
	-- OtQbg
	name
	)
	return self:getNodeManager():getChildByName(name)
end


function Node:getChildren()
	-- qvfƎ擾
	return self:getNodeManager().m_list
end


function Node:getChildrenCount()
	-- qvf̐擾
	return #self:getNodeManager().m_list
end


function Node:addEntries(
	mode,
	entry
	)
	
	table.insert(
		self.m_entries,
		{
			mode,
			entry
		}
	)
end


function Node:registerEntries()
	
	
	if self:getScene() and #self.m_entries > 0 then
		
		for i, entry in ipairs(self.m_entries) do
			if entry[1] == "schedule" then
				self:getScene():registerSchedule(self, entry[2])
			elseif entry[1] == "event" then
				if iskindof(entry[2], "cc.EventMouse") then
					self:getScene():registerEventMouseListener(self, entry[2])
				elseif iskindof(entry[2], "cc.EventKeyboard") then
					self:getScene():registerEventKeyboardListener(self, entry[2])
				end
			end
		end
		self.m_entries = {}
	end
end


function Node:scheduleUpdate(
	schedule
	)
	schedule = schedule or self.update
	self:addEntries("schedule", schedule)
end

function Node:addEventListener(
	event
	)
	
	self:addEntries("event", event)
end


function Node:isHover(
	-- zo[Ԃǂij
	location
	)
	local	parent = self:getParent()
	if parent == nil then return true end
	if cc.rectContainsPoint(self:getBoundingBox(), location) == false then return false end
	local	list = parent:getNodeManager().m_list
	for i = #list, 1, -1 do
		local	node = list[i]
		if node == self then
			-- ezo[
			return parent:isHover(location)
		elseif node:isVisible() then
			local	boundingBox = node:getBoundingBox()
			if cc.rectContainsPoint(boundingBox, location) then
				return false
			end
		end
	end
	return true
end


function Node:isLocalZOrderForeground(
	-- eɑ΂čőOʂɋ邩ij
	)
	local	parent = self:getParent()
	if parent then
		local	list = parent:getNodeManager().m_list
		for i = #list, 1, -1 do
			local	node = list[i]
			if node:isVisible() then
				return self == node
			end
		end
--		if list[#list] then
--			return self == list[#list]
--		end
	end
end


function Node:setLocalZOrderForeground(
	-- őOʂɈړij
	)
	if self:isLocalZOrderForeground() ~= true then
		if self:getParent() then
			local	children = self:getParent():getChildren()
			local	localZOrder = 0
			for i, child in ipairs(children) do
				if child:getLocalZOrder() > localZOrder then
					localZOrder = child:getLocalZOrder()
				end
			end
			self:setLocalZOrder(localZOrder + 1)
		end
	end
end


function Node:removeFromParent()
	-- ge菜
	self:getParent():getNodeManager():removeChild(self)
end


function Node:removeChild(
	-- Ԃ牺ĂYq菜
	child
	)
	self:getNodeManager():removeChild(child)
end


function Node:removeChildByTag(
	-- ^Ot̂폜
	tag
	)
	local	child = self:getNodeManager():getChildByTag(tag)
	if child then self:removeChild(child) end
end


function Node:removeChildByName(
	-- Ot̂폜
	name
	)
	local	child = self:getNodeManager():getChildByName(name)
	if child then self:removeChild(child) end
end

function Node:removeAllChildren()
	-- ׂč폜
	local	children = self:getChildren()
	for i, child in ipairs(children) do
		self:removeChild(child)
	end
end

function Node:draw()
	
	
	if self:getDirector() and self:getDirector():getDebugDraw() then
		-- fobO`
		
		local	drawSpace = self:getDrawSpace()
		local	x, y = drawSpace.x, drawSpace.y
		local	width, height = self:getContentSize().width, self:getContentSize().height
		love.graphics.polygon(
			"line",
			x, y,
			x + width, y,
			x + width, y + height,
			x, y + height
		)
	end
end


return Node
