-- 疑似シーン

local	Node = import(".Node")

local	EventMouseManager = import(".EventMouseManager")
local	EventKeyboardManager = import(".EventKeyboardManager")

local	Scene = {}
Scene = class("cc.Scene", Node)


function Scene:ctor(...)
	Node.ctor(self, ...)
	self.m_scene = self
	self.m_contentSize = cc.size(love.graphics.getWidth(), love.graphics.getHeight())
	
	self.m_eventMouseManager = EventMouseManager:create()
	self.m_eventKeyboardManager = EventKeyboardManager:create()
	self.m_schedules = {}
end

function Scene:createWithPhysics()
	-- 物理演算を構築しながら作成（仮）
	self:create()
	self.m_physicsWorld = love.physics.newWorld()
end


function Scene:registerSchedule(
	-- （仮）
	node,
	schedule
	)
	self.m_schedules[node] = {
		callback = schedule
	}
end

function Scene:removeSchedule(
	-- （仮）
	node
	)
	self.m_schedules[node] = nil
end


function Scene:registerEventMouseListener(
	-- （仮）
	node,
	eventMouseListener
	)
	
	self.m_eventMouseManager:addNode(node, eventMouseListener)
end


function Scene:removeEventMouseListener(
	-- （仮）
	node
	)
	self.m_eventMouseManager:removeNode(node)
end


function Scene:registerEventKeyboardListener(
	-- （仮）
	node,
	eventKeyboardListener
	)
	self.m_eventKeyboardManager:addNode(node, eventKeyboardListener)
end


function Scene:removeEventKeyboardListener(
	-- （仮）
	node
	)
	self.m_eventKeyboardManager:removeNode(node)
end



function Scene:onUpdate(
	dt
	)
	
	if self.m_physicsWorld then
		self.m_physicsWorld:update(dt)
	end
	
	-- ■スケジュール関連処理
	do
		for node, status in pairs(self.m_schedules) do
			status.callback(node, dt)
		end
	end
end


function Scene:mousemoved(...)
	if self.m_eventMouseManager then
		self.m_eventMouseManager:mousemoved(...)
	end
end


function Scene:mousepressed(...)
	if self.m_eventMouseManager then
		self.m_eventMouseManager:mousepressed(...)
	end
end


function Scene:mousereleased(...)
	if self.m_eventMouseManager then
		self.m_eventMouseManager:mousereleased(...)
	end
end


function Scene:wheelmoved(...)
	if self.m_eventMouseManager then
		self.m_eventMouseManager:wheelmoved(...)
	end
end

function Scene:keypressed(...)
	if self.m_eventKeyboardManager then
		self.m_eventKeyboardManager:keypressed(...)
	end
end


function Scene:keyreleased(...)
	if self.m_eventKeyboardManager then
		self.m_eventKeyboardManager:keyreleased(...)
	end
end

return Scene


