-- ^XvCg

local	Node = import(".Node")

local	Sprite = {}
Sprite = class("cc.Sprite", Node)


function Sprite:ctor(...)
	
	Node.ctor(self, ...)
	self:setAnchorPoint(cc.p(0.5, 0.5))
	
	local	args = { ... }
	local	filename = nil
	local	rect = nil
	
	self.m_texture = nil
	self.m_textureRect = nil
	self.m_textureQuad = nil
	self.m_rotation = 0
	self.m_scaleX = 1
	self.m_scaleY = 1
	
	if type(args[1]) == "string" then filename = args[1] end
	if type(args[2]) == "table" then rect = args[2] end
	
	if filename then
		self:initWithFile(filename, rect)
	end
end

function Sprite:initWithFile(
	filename,
	rect
	)
	local	width, height
	if type(filename) == "string" then
		self:setTexture(self:getDirector():loadSource("image", filename))
		width, height = self.m_texture:getWidth(), self.m_texture:getHeight()
		
		if rect and rect.width and rect.height then
			self.m_contentSize = cc.size(rect.width, rect.height)
		else
			self.m_contentSize = cc.size(width, height)
		end
		
	end
	if type(rect) == "table" then
		self:setTextureRect(rect, true)
	end
end


function Sprite:getTexture()
	return self.m_texture
end


function Sprite:setTexture(
	texture
	)
	if type(texture) == "string" then
		self:setTexture(self:getDirector():loadSource("image", texture))
	else
		self.m_texture = texture
	end
	if self.m_texture then
		self.m_texture:setFilter("nearest", "nearest")
	end
end


function Sprite:setRotation(
	-- pxZbgiangleŎw肵Ăˁj
	rotation
	)
	self.m_rotation = rotation
end


function Sprite:getRotation()
	return self.m_rotation
end


function Sprite:getTextureRect()
	return self.m_textureRect
end


function Sprite:setScale(
	scale
	)
	self.m_scaleX = scale
	self.m_scaleY = scale
end


function Sprite:setScaleX(
	scaleX
	)
	self.m_scaleX = scaleX
end


function Sprite:setScaleY(
	scaleY
	)
	self.m_scaleY = scaleY
end

function Sprite:setTextureRect(
	rect,
	resize
	)
	if self.m_texture then
		local	width, height = self.m_texture:getWidth(), self.m_texture:getHeight()
		
		self.m_textureRect = rect
		self.m_textureQuad = love.graphics.newQuad(rect.x, rect.y, rect.width, rect.height, width, height)
		
		if resize then
			self.m_contentSize = cc.size(rect.width or width, rect.height or height)
		end
	end
end





--function Sprite:draw()
--	
--	if self.m_texture then
--		-- 恄
--		local	drawSpace = self:getDrawSpace()
--		local	x, y = drawSpace.x, drawSpace.y
--		
--		local	width, height = self:getContentSize().width, self:getContentSize().height
----		local	width, height = self.m_texture:getWidth(), self.m_texture:getHeight()
--		
--		if self.m_textureQuad then
--			love.graphics.draw(self.m_texture,
--				self.m_textureQuad,
--				math.floor(x),
--				math.floor(y)
--			)
--		else
--			love.graphics.draw(self.m_texture,
--				math.floor(x),
--				math.floor(y)
--			)
--		end
--	end
--	
--	Node.draw(self)
--end


function Sprite:draw()
	
	if self.m_texture then
		local	drawSpace = self:getDrawSpace()
		local	x, y = math.floor(drawSpace.x), math.floor(drawSpace.y)
		local	width, height = self:getContentSize().width, self:getContentSize().height
		local	vx, vy, w, h
		if self.m_textureQuad then
			vx, vy, w, h = self.m_textureQuad:getViewport()
		else
			w, h = self.m_texture:getDimensions()
		end
		local	ap = self:getAnchorPoint()
		local	apip = self:getAnchorPointInPoints()
		local	rotate = math.angle2radian(self:getRotation())
		local	left = x + apip.x - w * self.m_scaleX * ap.x
		local	top  = y + apip.y - h * self.m_scaleY * ap.y
		
		love.graphics.push()
			-- ړ
			love.graphics.translate( math.floor(left + w * ap.x * self.m_scaleX),  math.floor(top + h * ap.y * self.m_scaleY))
			-- ]
			love.graphics.rotate(rotate)
			-- ړƂ߂
			love.graphics.translate(-math.floor(left + w * ap.x * self.m_scaleX), -math.floor(top + h * ap.y * self.m_scaleY))
			
			if self.m_textureQuad then
				love.graphics.draw(self.m_texture,
					self.m_textureQuad,
					left,
					top,
					0,
					self.m_scaleX,
					self.m_scaleY
				)
			else
				love.graphics.draw(self.m_texture,
					left,
					top,
					0,
					self.m_scaleX,
					self.m_scaleY
				)
			end
			
			-- ɖ߂
			love.graphics.origin()
		love.graphics.pop()
	end
	
	Node.draw(self)
end


return Sprite


--function draw_rota_graph(
--	-- OtBbN`
--	img,	--  C[Wf[^
--	...
--	-- (quad)	if[^j
--	-- x		XW
--	-- y		YW
--	-- scale	XP[
--	-- rotate	]iWAwj
--	-- xturn	E]
--	-- yturn	㉺]
--	)
--	local	argv = { ... }
--	local	shift = 0
--	local	quad = nil
--	if type(argv[1]) == "userdata" and type(argv[1].type) == "function" and argv[1]:type() == "Quad" then
--		quad = argv[1]
--		shift = 1
--	end
--	local	x = argv[1 + shift] or 0
--	local	y = argv[2 + shift] or 0
--	local	scale = argv[3 + shift] or 1.0
--	local	rotate = argv[4 + shift] or 0
--	local	xturn = argv[5 + shift] or false
--	local	yturn = argv[6 + shift] or false
--	local	left, top
--	local	vx, vy, w, h
--	local	sx = scale
--	local	sy = scale
--	
--	-- OtBbNTCY擾
--	if quad then
--		vx, vy, w, h = quad:getViewport()
--	else
--		w, h = img:getDimensions()
--	end
--	-- TCY荶Zo
--	left = x - w * sx * 0.5
--	top  = y - h * sy * 0.5
--	
--	-- `Jn
--	love.graphics.push()
--	do
--		if rotate ~= 0 then
--			-- ]
--			love.graphics.translate( math.floor(left + w * sx * 0.5),  math.floor(top + h * sy * 0.5))
--			love.graphics.rotate(rotate)
--			love.graphics.translate(-math.floor(left + w * sx * 0.5), -math.floor(top + h * sy * 0.5))
--		end
--		if xturn then
--			-- E]
--			left = left + w * sx
--			sx = sx * -1
--		end
--		if yturn then
--			-- ㉺]
--			top = top + h * sy
--			sy = sy * -1
--		end
--		if quad then
--			-- p`
--			love.graphics.draw(img, quad, left, top, 0, sx, sy)
--		else
--			-- 񕪊`
--			love.graphics.draw(img, left, top, 0, sx, sy)
--		end
--	end
--	love.graphics.pop()
--end
