-- ^th{^

--local	Node = import("..cc.Node")
local	Sprite = import("..cc.Sprite")
local	EventMouse = import("..cc.EventMouse")

local	Button = {}

Button = class("ccui.Button", Sprite)


function Button:ctor(...)
	Sprite.ctor(self)
	self:setAnchorPoint(cc.p(0.5, 0.5))
	local	args = { ... }
	
	local	normalimage = nil
	local	normalrect = nil
	local	selectedimage = nil
	local	selectedrect = nil
	
	self.m_textures = {}
	self.m_onClick = nil
	self.m_onEnter = nil
	self.m_onLeave = nil
	self.m_onUp = nil
	self.m_onDown = nil
	
	-- 
	if type(args[1]) == "table" then
		if type(args[1][1]) == "string" then
			normalimage = args[1][1]
		end
		if type(args[1][2]) ~= "nil" then
			normalrect = args[1][2]
		end
	elseif type(args[1]) == "string" then
		normalimage = args[1]
	end
	if type(args[2]) == "table" then
		if type(args[2][1]) == "string" then
			selectedimage = args[2][1]
		end
		if type(args[2][2]) ~= "nil" then
			selectedrect = args[2][2]
		end
	elseif type(args[2]) == "string" then
		selectedimage = args[1]
	end
	
	if normalimage then
		self:loadTextureNormal(normalimage, normalrect)
	end
	if selectedimage then
		self:loadTextureSelected(selectedimage, selectedrect)
	end
end

function Button:loadTextureNormal(
	filename,
	rect
	)
	local	texture
	
	if type(filename) == "string" then
		texture = self:getDirector():loadSource("image", filename)
--		texture = love.graphics.newImage(filename)
	end
	
	self.m_textures.normal = {
		texture = texture,
		rect = rect
	}
	
	self:swapTexture("normal")
	
--	if texture then
--		self.m_texture = texture
--		self.m_textureRect = rect
--	end
end


function Button:loadTextureSelected(
	filename,
	rect
	)
	local	texture
	if type(filename) == "string" then
		texture = self:getDirector():loadSource("image", filename)
--		texture = love.graphics.newImage(filename)
	end
	
	self.m_textures.selected = {
		texture = texture,
		rect = rect
	}
end


function Button:swapTexture(
	-- eLX`
	mode	-- normal
			-- selected
	)
	
	
	self:setTexture(nil)
	if mode == "normal" then
		if type(self.m_textures.normal) == "table" then
			self:setTexture(self.m_textures.normal.texture)
			if self.m_textures.normal.rect then self:setTextureRect(self.m_textures.normal.rect) end
		end
	elseif mode == "selected" then
		if type(self.m_textures.selected) == "table" then
			self:setTexture(self.m_textures.selected.texture)
			if self.m_textures.selected.rect then self:setTextureRect(self.m_textures.selected.rect) end
		end
	end
	
end

function Button:setMouseEnabled(
	-- }EXLɂ遨Listener֓o^
	mouseEnabled,
	argv
	)
	
	local	eventMouse = EventMouse:create()
	eventMouse.onMouseMove = self.onMouseMove
	eventMouse.onMouseUp = self.onMouseUp
	eventMouse.onMouseDown = self.onMouseDown
	
	if mouseEnabled then
		self:addEventListener(
			eventMouse
		)
	end
	
	if type(argv) == "function" then
		self.m_onClick = argv
	elseif type(argv) == "table" then
		if type(argv.onClick) == "function" then
			self.m_onClick = argv.onClick
		end
		if type(argv.onEnter) == "function" then
			self.m_onEnter = argv.onEnter
		end
		if type(argv.onLeave) == "function" then
			self.m_onLeave = argv.onLeave
		end
		if type(argv.onUp) == "function" then
			self.m_onUp = argv.onUp
		end
		if type(argv.onDown) == "function" then
			self.m_onDown = argv.onDown
		end
	end
end


function Button:onMouseMove(
	event
	)
	local	boundingBox = self:getBoundingBox()
	if self:isHover(event:getLocation()) and event:getStartLocation() then
		self:swapTexture("selected")
		if type(self.m_onEnter) == "function" then
			self.m_onEnter(self, event)
		end
	else
		self:swapTexture("normal")
		if type(self.m_onLeave) == "function" then
			self.m_onLeave(self, event)
		end
	end
end


function Button:onMouseUp(
	event
	)
	local	boundingBox = self:getBoundingBox()
	if self:isHover(event:getLocation()) then
		self:swapTexture("normal")
		if type(self.m_onLeave) == "function" then
			self.m_onLeave(self, event)
		end
		if type(self.m_onUp) == "function" then
			self.m_onUp(self, event)
		end
		if type(self.m_onClick) == "function" then
			self.m_onClick(self, event)
		end
		return true
	end
end

function Button:onMouseDown(
	event
	)
	local	boundingBox = self:getBoundingBox()
	if self:isHover(event:getLocation()) then
		self:swapTexture("selected")
		if type(self.m_onDown) == "function" then
			self.m_onDown(self, event)
		end
		if type(self.m_onEnter) == "function" then
			self.m_onEnter(self, event)
		end
		return true
	end
end



return Button