
local	Layout = import(".Layout")


local	ListView = {}
ListView = class("ccui.ListView", Layout)

function ListView:ctor(...)
	
	Layout.ctor(self, ...)
	
	self.m_direction = nil
	self.m_bounceEnabled = false
	self.m_backGroundImage = nil
	
	-- ReipCAEgp
	self.m_innerContainer = self:addChild(Layout:create())
	self.m_items = {}
	
	self.m_clippingType = 1
end


function ListView:getInnerContainer()
	return self.m_innerContainer
end


function ListView:setInnerContainerSize(
	size
	)
	self:getInnerContainer():setContentSize(size)
end


function ListView:getInnerContainerSize()
	return self:getInnerContainer():getContentSize()
end


function ListView:jumpToBottom()
	local	size = self:getContentSize()
	local	innerSize = self:getInnerContainerSize()
	if innerSize.height > size.height then
		self:getInnerContainer():setPosition(self:getInnerContainer():getPosition().x, size.height - innerSize.height)
	end
end

function ListView:jumpToTop()
	self:getInnerContainer():setPosition(self:getInnerContainer():getPosition().x, 0)
end

function ListView:jumpToLeft()
	self:getInnerContainer():setPosition(0, self:getInnerContainer():getPosition().y)
end

function ListView:jumpToRight()
	local	size = self:getContentSize()
	local	innerSize = self:getInnerContainerSize()
	if innerSize.width > size.width then
		self:getInnerContainer():setPosition(size.widh - innerSize.width, self:getInnerContainer():getPosition().y)
	end
end

function ListView:jumpToTopLeft()
	self:jumpToLeft()
	self:jumpToTop()
end

function ListView:jumpToTopRight()
	self:jumpToRight()
	self:jumpToTop()
end

function ListView:jumpToBottomLeft()
	self:jumpToLeft()
	self:jumpToBottom()
end

function ListView:jumpToBottomRight()
	self:jumpToRight()
	self:jumpToBottom()
end

function ListView:jumpToPercentVertical(
	percent
	)
	local	size = self:getContentSize()
	local	innerSize = self:getInnerContainerSize()
	if innerSize.height > size.height then
		self:getInnerContainer():setPosition(self:getInnerContainer():getPosition().x, (size.height - innerSize.height) * percent)
	else
		self:getInnerContainer():setPosition(self:getInnerContainer():getPosition().x, 0)
	end
end

function ListView:jumpToPercentHorizontal(
	percent
	)
	local	size = self:getContentSize()
	local	innerSize = self:getInnerContainerSize()
	if innerSize.width > size.width then
		self:getInnerContainer():setPosition((size.width - innerSize.width) * percent, self:getInnerContainer():getPosition().y)
	else
		self:getInnerContainer():setPosition(0, self:getInnerContainer():getPosition().y)
	end
end

function ListView:jumpToPercentBothDirection(
	percent
	)
	self:jumpToPercentVertical(percent.x)
	self:jumpToPercentHorizontal(percent.y)
end

function ListView:refreshView()
	-- tbV
	local	position = cc.p(0, 0)
	for i, item in ipairs(self.m_items) do
		local	itemSize = item:getContentSize()
		-- Ń^OtĂ
		item:setTag(i)
		item:setPosition(position.x, position.y)
		position.y = position.y + itemSize.height
		-- TCY
		if self:getInnerContainer():getContentSize().width < itemSize.width then
			self:getInnerContainer():setContentSize(cc.size(itemSize.width, self:getInnerContainer():getContentSize().height))
		end
		if self:getInnerContainer():getContentSize().height < position.y then
			self:getInnerContainer():setContentSize(cc.size(self:getInnerContainer():getContentSize().width, position.y))
		end
	end
end


function ListView:pushBackCustomItem(item)
	table.insert(self.m_items, item)
	self:getInnerContainer():addChild(item)
	self:refreshView()
end


function ListView:removeItem(index)
	table.remove(self.m_items, index)
	self:removeChildByTag(index)
	self:refreshView()
end


function ListView:insertCustomItem(item, index)
	table.insert(self.m_items, index)
	self:removeChildByTag(index)
	self:refreshView()
end


function ListView:removeLastItem()
	local	index = #self.m_items
	table.remove(self.m_items)
	self:removeChildByTag(index)
	self:refreshView()
end


function ListView:removeAllItems()
	self.m_items = {}
	self:getInnerContainer():removeAllChildren()
	self:getInnerContainer():cleanup()
	self:refreshView()
end

function ListView:getItem(index)
	return self.m_items[index]
end


function ListView:getItems()
	return self.m_items
end


return ListView
